<?php
/**
 * Settings: General.
 *
 * @since 2.2.0
 */

namespace Themesquad\WC_Amazon_S3_Storage\Admin\Settings;

defined( 'ABSPATH' ) || exit;

use Themesquad\WC_Amazon_S3_Storage\Internal\Admin\Settings\Abstracts\Settings_API;
use Themesquad\WC_Amazon_S3_Storage\Utilities\Admin_Utils;

/**
 * General settings class.
 */
class General extends Settings_API {

	/**
	 * Save the settings individually or grouped in a single option.
	 *
	 * @var bool
	 */
	public $save_individually = false;

	/**
	 * Constructor.
	 *
	 * @since 2.2.0
	 */
	public function __construct() {
		$this->id = 'settings';
	}

	/**
	 * Gets the form title.
	 *
	 * @since 2.2.0
	 *
	 * @return string
	 */
	public function get_form_title() {
		return _x( 'Amazon S3 Storage', 'settings page title', 'woocommerce-amazon-s3-storage' );
	}

	/**
	 * Gets the form description.
	 *
	 * @since 2.2.0
	 *
	 * @return string
	 */
	public function get_form_description() {
		return _x( 'An integration for utilizing the Amazon Web Services to store your downloadable products.', 'settings page description', 'woocommerce-amazon-s3-storage' );
	}

	/**
	 * Gets the name of the option in the WP DB.
	 *
	 * @since 2.2.0
	 *
	 * @param string $setting Optional. Setting key.
	 * @return string
	 */
	public function get_option_key( $setting = '' ) {
		return 'woo_amazon_s3_storage';
	}

	/**
	 * Initialise form fields.
	 *
	 * @since 2.2.0
	 */
	public function init_form_fields() {
		$this->form_fields = array(
			'amazon_access_key'    => array(
				'type'     => 'password',
				'title'    => _x( 'Access Key ID', 'setting title', 'woocommerce-amazon-s3-storage' ),
				'desc_tip' => _x( 'Your Amazon Web Services access key ID.', 'setting desc', 'woocommerce-amazon-s3-storage' ),
			),
			'amazon_access_secret' => array(
				'type'     => 'password',
				'title'    => _x( 'Secret Access Key', 'setting title', 'woocommerce-amazon-s3-storage' ),
				'desc_tip' => _x( 'Your Amazon Web Services secret access key', 'setting desc', 'woocommerce-amazon-s3-storage' ),
			),
		);

		$settings = get_option( $this->get_option_key(), array() );

		// Display the rest of the fields when the credentials are entered.
		if ( empty( $settings['amazon_access_key'] ) || empty( $settings['amazon_access_secret'] ) ) {
			return;
		}

		$additional_settings = array(
			'amazon_url_period' => array(
				'type'              => 'number',
				'title'             => _x( 'URL Valid Period', 'setting title', 'woocommerce-amazon-s3-storage' ),
				'desc_tip'          => _x( 'Time in minutes that the URL is valid for downloading.', 'setting desc', 'woocommerce-amazon-s3-storage' ),
				'description'       => _x( 'The maximum allowed value is seven days.', 'setting desc', 'woocommerce-amazon-s3-storage' ),
				'placeholder'       => 1,
				'css'               => 'max-width: 100px',
				'custom_attributes' => array(
					'min'  => 1,
					'max'  => 10080, // 7 days in minutes (maximum allowed by AWS).
					'step' => 1,
				),
			),
			'default_bucket'    => array(
				'type'        => 'text',
				'title'       => _x( 'Bucket Name', 'setting title', 'woocommerce-amazon-s3-storage' ),
				'desc_tip'    => _x( 'The default bucket name.', 'setting desc', 'woocommerce-amazon-s3-storage' ),
				'description' => _x( 'Use the shortcode parameter <code>bucket</code> to overwrite this value.', 'setting desc', 'woocommerce-amazon-s3-storage' ),
			),
			'default_region'    => array(
				'type'        => 'select',
				'title'       => _x( 'S3 Region', 'setting title', 'woocommerce-amazon-s3-storage' ),
				'desc_tip'    => _x( 'The default S3 region.', 'setting desc', 'woocommerce-amazon-s3-storage' ),
				'description' => _x( 'Use the shortcode parameter <code>region</code> to overwrite this value.', 'setting desc', 'woocommerce-amazon-s3-storage' ),
				'class'       => 'wc-enhanced-select',
				'options'     => array( '' => __( 'Select a region', 'woocommerce-amazon-s3-storage' ) ) + $this->get_region_options(),
			),
		);

		$this->form_fields = array_merge( $this->form_fields, $additional_settings );
	}

	/**
	 * Gets the options for the S3 Regions select field.
	 *
	 * @since 2.3.0
	 *
	 * @return array
	 */
	protected function get_region_options() {
		return Admin_Utils::get_region_options();
	}
}
