<?php
/**
 * Handles the plugin activation and install.
 *
 * @since 2.5.0
 */

namespace Themesquad\WC_Amazon_S3_Storage\Database;

defined( 'ABSPATH' ) || exit;

use Themesquad\WC_Amazon_S3_Storage\Admin\Notices;
use Themesquad\WC_Amazon_S3_Storage\Utilities\Admin_Utils;

/**
 * Class Installer.
 */
class Installer {

	/**
	 * Plugin updater.
	 *
	 * @var Updater
	 */
	private static $updater;

	/**
	 * Database updates that need to be run per version.
	 *
	 * @var array
	 */
	private static $db_updates = array(
		'2.0.0' => 'Themesquad\WC_Amazon_S3_Storage\Database\Updates\Update_200_Migrate_Product_Files',
		'2.1.5' => 'Themesquad\WC_Amazon_S3_Storage\Database\Updates\Update_215_Expire_Region_Transients',
	);

	/**
	 * Init installation.
	 *
	 * @since 2.5.0
	 */
	public static function init() {
		register_activation_hook( WC_AMAZON_S3_STORAGE_FILE, array( __CLASS__, 'install' ) );

		add_action( 'init', array( __CLASS__, 'check_version' ), 5 );
		add_action( 'init', array( __CLASS__, 'init_updater' ), 7 );
		add_action( 'admin_init', array( __CLASS__, 'install_actions' ) );
		add_action( 'admin_print_styles', array( __CLASS__, 'add_notices' ) );
		add_action( 'wc_amazon_s3_storage_updater_complete', array( __CLASS__, 'updated' ) );
	}

	/**
	 * Get the database updates.
	 *
	 * @since 2.5.0
	 *
	 * @return array
	 */
	public static function get_db_updates() {
		return self::$db_updates;
	}

	/**
	 * Gets the installed version of the plugin or for the specified component.
	 *
	 * @since 2.5.0
	 *
	 * @param string $component Optional. The plugin component. Default empty. Accepts: 'db'.
	 * @return string|null
	 */
	public static function get_version( $component = '' ) {
		return get_option( self::get_version_option_key( $component ), null );
	}

	/**
	 * Updates the version of the plugin or the specified component.
	 *
	 * @since 2.5.0
	 *
	 * @param string $version   The new version.
	 * @param string $component Optional. The plugin component. Default empty. Accepts: 'db'.
	 */
	public static function update_version( $version, $component = '' ) {
		update_option( self::get_version_option_key( $component ), $version );
	}

	/**
	 * Gets the option key that stores the version of the plugin or the specified component.
	 *
	 * @since 2.5.0
	 *
	 * @param string $component Optional. The plugin component. Default empty. Accepts: 'db'.
	 * @return string
	 */
	protected static function get_version_option_key( $component = '' ) {
		$suffix = ( $component ? "_$component" : '' ) . '_version';

		return "wc_amazon_s3_storage{$suffix}";
	}

	/**
	 * Checks the plugin version and run the installation process if necessary.
	 *
	 * This check is done on all requests and runs if the versions do not match.
	 *
	 * @since 2.5.0
	 */
	public static function check_version() {
		if ( ! defined( 'IFRAME_REQUEST' ) && version_compare( self::get_version(), WC_AMAZON_S3_STORAGE_VERSION, '<' ) ) {
			self::install();
		}
	}

	/**
	 * Initializes the plugin updater.
	 *
	 * @since 2.5.0
	 */
	public static function init_updater() {
		self::$updater = new Updater();
	}

	/**
	 * Install actions when an update button is clicked within the admin area.
	 *
	 * @since 2.5.0
	 */
	public static function install_actions() {
		if ( ! empty( $_GET['do_update_wc_amazon_s3_storage'] ) ) {
			check_admin_referer( 'wc_amazon_s3_storage_db_update', 'wc_amazon_s3_storage_db_update_nonce' );
			self::update();
		}

		if ( ! empty( $_GET['force_update_wc_amazon_s3_storage'] ) ) {
			check_admin_referer( 'wc_amazon_s3_storage_force_db_update', 'wc_amazon_s3_storage_force_db_update_nonce' );
			self::$updater->force_process();
			wp_safe_redirect( Admin_Utils::get_settings_url() );
			exit;
		}
	}

	/**
	 * Removes any notices added to admin.
	 *
	 * @since 2.5.0
	 */
	private static function remove_notices() {
		// Remove not dismissed notice from a previous update.
		Notices::remove_notice( 'wc_amazon_s3_storage_updated' );
	}

	/**
	 * Add installer/updater notices + styles if needed.
	 *
	 * @since 2.5.0
	 */
	public static function add_notices() {
		// Dismiss the 'extension updated' notice before displaying more notices.
		if ( Notices::has_notice( 'wc_amazon_s3_storage_updated' ) ) {
			return;
		}

		if ( self::needs_db_update() ) {
			if ( self::$updater->is_updating() || ! empty( $_GET['do_update_wc_amazon_s3_storage'] ) ) { // phpcs:ignore WordPress.Security.NonceVerification
				Notices::add_notice( 'updating' );
			} else {
				Notices::add_notice( 'update' );
			}
		}
	}

	/**
	 * Init installation.
	 *
	 * @since 2.5.0
	 */
	public static function install() {
		if ( ! is_blog_installed() ) {
			return;
		}

		// Check if we are not already running the installation process.
		if ( 'yes' === get_transient( 'wc_amazon_s3_storage_installing' ) ) {
			return;
		}

		// Add a transient to indicate that we are running the installation process.
		set_transient( 'wc_amazon_s3_storage_installing', 'yes', MINUTE_IN_SECONDS * 10 );

		self::remove_notices();
		self::update_version( WC_AMAZON_S3_STORAGE_VERSION );
		self::maybe_update_db();

		// Installation finished.
		delete_transient( 'wc_amazon_s3_storage_installing' );

		flush_rewrite_rules();

		/**
		 * Fires when the plugin installation finished.
		 *
		 * @since 2.5.0
		 */
		do_action( 'wc_amazon_s3_storage_installed' );
	}

	/**
	 * Update the database if necessary.
	 *
	 * @since 2.5.0
	 */
	private static function maybe_update_db() {
		if ( ! self::needs_db_update() ) {
			self::update_version( WC_AMAZON_S3_STORAGE_VERSION, 'db' );
		}
	}

	/**
	 * Get if the database needs to be updated or not.
	 *
	 * @since 2.5.0
	 *
	 * @return bool
	 */
	private static function needs_db_update() {
		$db_version = self::get_version( 'db' );

		if ( ! $db_version ) {
			return false;
		}

		$updates = array_keys( self::get_db_updates() );

		usort( $updates, 'version_compare' );

		return version_compare( $db_version, end( $updates ), '<' );
	}

	/**
	 * Pushes all needed database updates to the queue for processing.
	 *
	 * @since 2.5.0
	 */
	private static function update() {
		$db_version    = self::get_version( 'db' );
		$db_updates    = self::get_db_updates();
		$update_queued = false;

		foreach ( $db_updates as $version => $updates ) {
			if ( version_compare( $db_version, $version, '<' ) ) {
				$updates = (array) $updates;

				foreach ( $updates as $update ) {
					self::$updater->push_to_queue( compact( 'update' ) );
					$update_queued = true;
				}

				// Update db version.
				self::$updater->push_to_queue(
					array(
						'update' => '\Themesquad\WC_Amazon_S3_Storage\Database\Updates\Update_DB_Version',
						'args'   => compact( 'version' ),
					)
				);
			}
		}

		if ( $update_queued ) {
			self::$updater->save()->dispatch();
		}
	}

	/**
	 * Database updated.
	 *
	 * @since 2.5.0
	 */
	public static function updated() {
		self::update_version( WC_AMAZON_S3_STORAGE_VERSION, 'db' );

		Notices::add_dismiss_notice(
			'wc_amazon_s3_storage_updated',
			__( 'WooCommerce Amazon S3 Storage update completed. Thank you for updating to the latest version!', 'woocommerce-amazon-s3-storage' )
		);
	}
}
