<?php
/**
 * WooCommerce Amazon S3 Storage.
 *
 * @since 2.2.0
 */

namespace Themesquad\WC_Amazon_S3_Storage;

defined( 'ABSPATH' ) || exit;

use Themesquad\WC_Amazon_S3_Storage\Admin\Admin;
use Themesquad\WC_Amazon_S3_Storage\Database\Installer;
use Themesquad\WC_Amazon_S3_Storage\Internal\Traits\Singleton;

/**
 * Plugin class.
 */
class Plugin {

	use Singleton;

	/**
	 * Constructor.
	 *
	 * @since 2.2.0
	 */
	protected function __construct() {
		$this->define_constants();
		$this->init();
	}

	/**
	 * Define constants.
	 *
	 * @since 2.2.0
	 */
	private function define_constants() {
		$this->define( 'WC_AMAZON_S3_STORAGE_VERSION', '2.5.1' );
		$this->define( 'WC_AMAZON_S3_STORAGE_PATH', plugin_dir_path( WC_AMAZON_S3_STORAGE_FILE ) );
		$this->define( 'WC_AMAZON_S3_STORAGE_URL', plugin_dir_url( WC_AMAZON_S3_STORAGE_FILE ) );
		$this->define( 'WC_AMAZON_S3_STORAGE_BASENAME', plugin_basename( WC_AMAZON_S3_STORAGE_FILE ) );
	}

	/**
	 * Define constant if not already set.
	 *
	 * @since 2.2.0
	 *
	 * @param string      $name  The constant name.
	 * @param string|bool $value The constant value.
	 */
	private function define( $name, $value ) {
		if ( ! defined( $name ) ) {
			define( $name, $value );
		}
	}

	/**
	 * Init plugin.
	 *
	 * @since 2.2.0
	 */
	private function init() {
		add_action( 'before_woocommerce_init', array( $this, 'declare_compatibility' ) );
		add_action( 'init', array( $this, 'load_plugin_textdomain' ) );
		add_action( 'woocommerce_loaded', array( $this, 'wc_loaded' ) );
		add_filter( 'woocommerce_integrations', array( $this, 'register_integration' ) );

		Installer::init();

		if ( is_admin() ) {
			Admin::init();
		}
	}

	/**
	 * Declares compatibility with the WC features.
	 *
	 * @since 2.4.0
	 */
	public function declare_compatibility() {
		// Compatible with the 'High-Performance Order Storage' feature.
		if ( class_exists( '\Automattic\WooCommerce\Utilities\FeaturesUtil' ) ) {
			\Automattic\WooCommerce\Utilities\FeaturesUtil::declare_compatibility( 'custom_order_tables', WC_AMAZON_S3_STORAGE_FILE, true );
		}
	}

	/**
	 * Load plugin text domain.
	 *
	 * @since 2.2.0
	 */
	public function load_plugin_textdomain() {
		load_plugin_textdomain( 'woocommerce-amazon-s3-storage', false, dirname( WC_AMAZON_S3_STORAGE_BASENAME ) . '/languages' );
	}

	/**
	 * Load more functionality after WC has been initialized.
	 *
	 * @since 2.5.0
	 */
	public function wc_loaded() {
		if ( class_exists( 'WC_Abstract_Privacy' ) ) {
			new Privacy();
		}
	}

	/**
	 * Registers the integration.
	 *
	 * @since 2.2.0
	 *
	 * @param array $integrations Array of integration instances.
	 * @return array
	 */
	public function register_integration( $integrations ) {
		$integrations[] = '\Themesquad\WC_Amazon_S3_Storage\Admin\Settings\Integration';

		return $integrations;
	}
}
